# encoding: utf-8
"""
@author: Yuxian Meng
@contact: yuxian_meng@shannonai.com

@version: 1.0
@file: glyph_embedding
@time: 2020/8/4 15:04

"""

from typing import List

import numpy as np
import torch
from torch import nn


class GlyphEmbedding(nn.Module):
    """Glyph2Image Embedding"""

    def __init__(self, font_npy_files: List[str]):
        super(GlyphEmbedding, self).__init__()
        font_arrays = [
            np.load(np_file).astype(np.float32) for np_file in font_npy_files
        ]
        self.vocab_size = font_arrays[0].shape[0]
        self.font_num = len(font_arrays)
        self.font_size = font_arrays[0].shape[-1]
        # N, C, H, W
        font_array = np.stack(font_arrays, axis=1)
        self.embedding = nn.Embedding(
            num_embeddings=self.vocab_size,
            embedding_dim=self.font_size ** 2 * self.font_num,
            _weight=torch.from_numpy(font_array.reshape([self.vocab_size, -1]))
        )

    def forward(self, input_ids):
        """
            get glyph images for batch inputs
        Args:
            input_ids: [batch, sentence_length]
        Returns:
            images: [batch, sentence_length, self.font_num*self.font_size*self.font_size]
        """
        # return self.embedding(input_ids).view([-1, self.font_num, self.font_size, self.font_size])
        return self.embedding(input_ids)



import torch
from glycebert import GlyceBertTokenizerFast, GlyceBertForMaskedLM

pretrained_tokenizer_name = "junnyu/ChineseBERT-base"
pretrained_model_name = "shannoai/ChineseBERT-base"

tokenizer = GlyceBertTokenizerFast.from_pretrained(pretrained_tokenizer_name)
chinese_bert = GlyceBertForMaskedLM.from_pretrained(pretrained_model_name)

text = "北京是[MASK]国的首都。"
inputs = tokenizer(text, return_tensors="pt")
maskpos = 4
with torch.no_grad():
    o = chinese_bert(**inputs)
    value, index = o.logits.softmax(-1)[0, maskpos].topk(10)

pred_tokens = tokenizer.convert_ids_to_tokens(index.tolist())
pred_values = value.tolist()

outputs = []
for t, p in zip(pred_tokens, pred_values):
    outputs.append(f"{t}|{round(p,4)}")
print(outputs)
# ['中|0.711', '我|0.2488', '祖|0.016', '法|0.0057', '美|0.0048', '全|0.0042', '韩|0.0015', '英|0.0011', '两|0.0008', '王|0.0006']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# author: xiaoya li
# first create: 2021.01.25
# files: nn_modules.py
#

import torch.nn as nn

class BertMLP(nn.Module):
    def __init__(self, config,):
        super().__init__()
        self.dense_layer = nn.Linear(config.hidden_size, config.hidden_size)
        self.dense_to_labels_layer = nn.Linear(config.hidden_size, config.num_labels)
        self.activation = nn.Tanh()

    def forward(self, sequence_hidden_states):
        sequence_output = self.dense_layer(sequence_hidden_states)
        sequence_output = self.activation(sequence_output)
        sequence_output = self.dense_to_labels_layer(sequence_output)
        return sequence_output



from glycebert.tokenization_glycebert_fast import GlyceBertTokenizerFast
from glycebert.modeling_glycebert import (
    GlyceBertModel,
    GlyceBertForMaskedLM,
    GlyceBertForSequenceClassification,
    GlyceBertForQuestionAnswering,
    GlyceBertForTokenClassification,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@file  : pinyin.py
@author: zijun
@contact : zijun_sun@shannonai.com
@date  : 2020/8/16 14:45
@version: 1.0
@desc  : pinyin embedding
"""
import json
import os

from torch import nn
from torch.nn import functional as F


class PinyinEmbedding(nn.Module):
    def __init__(self, embedding_size: int, pinyin_out_dim: int, config_path):
        """
            Pinyin Embedding Module
        Args:
            embedding_size: the size of each embedding vector
            pinyin_out_dim: kernel number of conv
        """
        super(PinyinEmbedding, self).__init__()
        with open(os.path.join(config_path, "pinyin_map.json")) as fin:
            pinyin_dict = json.load(fin)
        self.pinyin_out_dim = pinyin_out_dim
        self.embedding = nn.Embedding(len(pinyin_dict["idx2char"]), embedding_size)
        self.conv = nn.Conv1d(
            in_channels=embedding_size,
            out_channels=self.pinyin_out_dim,
            kernel_size=2,
            stride=1,
            padding=0,
        )

    def forward(self, pinyin_ids):
        """
        Args:
            pinyin_ids: (bs*sentence_length*pinyin_locs)

        Returns:
            pinyin_embed: (bs,sentence_length,pinyin_out_dim)
        """
        # input pinyin ids for 1-D conv
        embed = self.embedding(
            pinyin_ids
        )  # [bs,sentence_length,pinyin_locs,embed_size]

        bs, sentence_length, pinyin_locs, embed_size = embed.shape
        view_embed = embed.view(
            -1, pinyin_locs, embed_size
        )  # [(bs*sentence_length),pinyin_locs,embed_size]
        input_embed = view_embed.permute(
            0, 2, 1
        )  # [(bs*sentence_length), embed_size, pinyin_locs]
        # conv + max_pooling
        pinyin_conv = self.conv(input_embed)  # [(bs*sentence_length),pinyin_out_dim,H]
        pinyin_embed = F.max_pool1d(
            pinyin_conv, pinyin_conv.shape[-1]
        )  # [(bs*sentence_length),pinyin_out_dim,1]
        return pinyin_embed.view(
            bs, sentence_length, self.pinyin_out_dim
        )  # [bs,sentence_length,pinyin_out_dim]



from setuptools import find_packages, setup

setup(
    name="glycebert",
    package_dir={"": "src"},
    packages=find_packages("src"),
    version="0.0.1",
    license="MIT",
    description="GlyceBert_pytorch",
    author="Jun Yu",
    author_email="573009727@qq.com",
    url="https://github.com/JunnYu/GlyceBert_pytorch",
    keywords=["GlyceBert", "pytorch", "tf2.0"],
    install_requires=["transformers>=4.7.0", "pypinyin"],
)
